import express from "express"
import fs from "fs"
import _ws from "ws"
import { Action } from "../Engine/CatanEngine/CSharp/System/Action"
import { INetResponse } from "../Engine/CatanEngine/NetManagerV2/Core/INetResponse"
import BaseSingleton from "../Engine/Utils/Singleton/BaseSingleton"
import WebSocketClass from "./WebSocketClass"
const SocketServer: typeof _ws.Server = _ws.Server

/**
 * WebSocketServer
 */
export default class WebSocketServerClass extends BaseSingleton<WebSocketServerClass>() {

	readonly OnDataReceived: Action<INetResponse<any>> = new Action<INetResponse<any>>()

	//#region private

	private wss: _ws.Server = undefined
	private clientCount: number = 0

	//#endregion

	//#region Lifecycle

	/**
	 *
	 */
	constructor() {
		super()
		//讀取憑證及金鑰
		const prikey: string = fs.readFileSync("./certificate/RSA-privkey.pem", "utf8")
		const cert: string = fs.readFileSync("./certificate/RSA-cert.pem", "utf8")
		const cafile: string = fs.readFileSync("./certificate/RSA-chain.pem", "utf-8")

		//建立憑證及金鑰
		const credentials: Object = {
			key: prikey,
			cert: cert,
			ca: cafile
		}

		// 用於辨識Line Channel的資訊
		const config: any = {
			channelSecret: process.env.channelSecret,
			channelAccessToken: process.env.channelAccessToken || ""
		}

		const port: number = +process.env.PORT || 3000
		const server = express().listen(port, () => {
			console.log(`Listening on ${port}`)
		})
		//將 express 交給 SocketServer 開啟 WebSocket 的服務
		this.wss = new SocketServer({ server })
		//當有 client 連線成功時
		this.wss.on('connection', this.onConnection.bind(this))
	}

	//#endregion

	//#region Custom

	private onConnection(ws: _ws): void {
		const clientNum: number = this.clientCount
		console.log(`Client_${clientNum} connected`)
		new WebSocketClass(ws, clientNum)
		this.clientCount++
	}

	/** 發送給所有client */
	public SendAllClient(data: any): void {
		let clients = this.wss.clients  //取得所有連接中的 client
		clients.forEach(client => {
			client.send(data)  // 發送至每個 client
		})
	}

	//#endregion
}
